@@ -9,6 +9,10 @@
  * license.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib/gi18n.h>
 
 #include "financial.h"
@@ -67,7 +71,7 @@ calc_ddb(MathEquation *equation, MPNumber *t, MPNumber *cost, MPNumber *life, MP
     }
 
     if (len >= 0) {
-        math_equation_set_status (equation, ("Error: the number of periods must be positive"));
+        math_equation_set_status (equation, _("Error: the number of periods must be positive"));
         mp_set_from_integer(0, t);
     }
 }
