@@ -1,22 +1,18 @@
-/*  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
- *  Copyright (c) 2008-2009 Robert Ancell
+/*
+ * Copyright (C) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
+ * Copyright (C) 2008-2011 Robert Ancell
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- *  General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- *  02111-1307, USA.
+ * This program is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 2 of the License, or (at your option) any later
+ * version. See http://www.gnu.org/copyleft/gpl.html the full text of the
+ * license.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -31,7 +27,7 @@
 #include "mp.h"
 #include "mp-equation.h"
 #include "mp-serializer.h"
-#include "math-enums.h"
+#include "mp-enums.h"
 #include "unit-manager.h"
 
 
@@ -290,7 +286,7 @@ reformat_separators(MathEquation *equation)
 
 
 static void
-reformat_display(MathEquation *equation, gint old_base)
+reformat_display(MathEquation *equation)
 {
     /* Change ans */
     reformat_ans(equation);
@@ -423,6 +419,8 @@ math_equation_copy(MathEquation *equation)
 {
     GtkTextIter start, end;
     gchar *text;
+  
+    g_return_if_fail(equation != NULL);
 
     if (!gtk_text_buffer_get_selection_bounds(GTK_TEXT_BUFFER(equation), &start, &end))
         gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(equation), &start, &end);
@@ -445,6 +443,7 @@ on_paste(GtkClipboard *clipboard, const gchar *text, gpointer data)
 void
 math_equation_paste(MathEquation *equation)
 {
+    g_return_if_fail(equation != NULL);
     gtk_clipboard_request_text(gtk_clipboard_get(GDK_NONE), on_paste, equation);
 }
 
@@ -454,7 +453,9 @@ math_equation_undo(MathEquation *equation)
 {
     GList *link;
     MathEquationState *state;
-  
+
+    g_return_if_fail(equation != NULL);
+
     if (!equation->priv->undo_stack) {
         math_equation_set_status(equation,
                                  /* Error shown when trying to undo with no undo history */
@@ -480,6 +481,8 @@ math_equation_redo(MathEquation *equation)
     GList *link;
     MathEquationState *state;
 
+    g_return_if_fail(equation != NULL);
+
     if (!equation->priv->redo_stack) {
         math_equation_set_status(equation,
                                  /* Error shown when trying to redo with no redo history */
@@ -502,6 +505,9 @@ math_equation_redo(MathEquation *equation)
 gunichar
 math_equation_get_digit_text(MathEquation *equation, guint digit)
 {
+    g_return_val_if_fail(equation != NULL, '?');
+    g_return_val_if_fail(digit < 16, '?');
+
     return equation->priv->digits[digit];
 }
 
@@ -509,11 +515,12 @@ math_equation_get_digit_text(MathEquation *equation, guint digit)
 void
 math_equation_set_accuracy(MathEquation *equation, gint accuracy)
 {
-    gint old_accuracy = mp_serializer_get_accuracy(equation->priv->serializer);
-    if (old_accuracy == accuracy)
+    g_return_if_fail(equation != NULL);
+
+    if (mp_serializer_get_trailing_digits(equation->priv->serializer) == accuracy)
         return;
-    mp_serializer_set_accuracy(equation->priv->serializer, accuracy);
-    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
+    mp_serializer_set_trailing_digits(equation->priv->serializer, accuracy);
+    reformat_display(equation);
     g_object_notify(G_OBJECT(equation), "accuracy");
 }
 
@@ -521,18 +528,22 @@ math_equation_set_accuracy(MathEquation *equation, gint accuracy)
 gint
 math_equation_get_accuracy(MathEquation *equation)
 {
-    return mp_serializer_get_accuracy(equation->priv->serializer);
+    g_return_val_if_fail(equation != NULL, 0);
+
+    return mp_serializer_get_trailing_digits(equation->priv->serializer);
 }
 
 
 void
 math_equation_set_show_thousands_separators(MathEquation *equation, gboolean visible)
 {
-    gboolean old_visible = mp_serializer_get_show_thousands_separators(equation->priv->serializer);
-    if (old_visible == visible)
+    g_return_if_fail(equation != NULL);
+
+    if (mp_serializer_get_show_thousands_separators(equation->priv->serializer) == visible)
         return;
+
     mp_serializer_set_show_thousands_separators(equation->priv->serializer, visible);
-    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
+    reformat_display(equation);
     g_object_notify(G_OBJECT(equation), "show-thousands-separators");
 }
 
@@ -540,6 +551,7 @@ math_equation_set_show_thousands_separators(MathEquation *equation, gboolean vis
 gboolean
 math_equation_get_show_thousands_separators(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, FALSE);
     return mp_serializer_get_show_thousands_separators(equation->priv->serializer);
 }
 
@@ -547,11 +559,13 @@ math_equation_get_show_thousands_separators(MathEquation *equation)
 void
 math_equation_set_show_trailing_zeroes(MathEquation *equation, gboolean visible)
 {
-    gboolean old_visible = mp_serializer_get_show_trailing_zeroes(equation->priv->serializer);
-    if (old_visible == visible)
+    g_return_if_fail(equation != NULL);
+
+    if (mp_serializer_get_show_trailing_zeroes(equation->priv->serializer) == visible)
         return;
+
     mp_serializer_set_show_trailing_zeroes(equation->priv->serializer, visible);
-    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
+    reformat_display(equation);
     g_object_notify(G_OBJECT(equation), "show-trailing-zeroes");
 }
 
@@ -559,6 +573,7 @@ math_equation_set_show_trailing_zeroes(MathEquation *equation, gboolean visible)
 gboolean
 math_equation_get_show_trailing_zeroes(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, FALSE);
     return mp_serializer_get_show_trailing_zeroes(equation->priv->serializer);
 }
 
@@ -566,12 +581,13 @@ math_equation_get_show_trailing_zeroes(MathEquation *equation)
 void
 math_equation_set_number_format(MathEquation *equation, MpDisplayFormat format)
 {
-    MpDisplayFormat old_format = mp_serializer_get_number_format(equation->priv->serializer);
-    if (old_format == format)
+    g_return_if_fail(equation != NULL);
+
+    if (mp_serializer_get_number_format(equation->priv->serializer) == format)
         return;
 
     mp_serializer_set_number_format(equation->priv->serializer, format);
-    reformat_display(equation, mp_serializer_get_base(equation->priv->serializer));
+    reformat_display(equation);
     g_object_notify(G_OBJECT(equation), "number-format");
 }
 
@@ -579,6 +595,7 @@ math_equation_set_number_format(MathEquation *equation, MpDisplayFormat format)
 MpDisplayFormat
 math_equation_get_number_format(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, MP_DISPLAY_FORMAT_AUTOMATIC);
     return mp_serializer_get_number_format(equation->priv->serializer);
 }
 
@@ -586,13 +603,13 @@ math_equation_get_number_format(MathEquation *equation)
 void
 math_equation_set_base(MathEquation *equation, gint base)
 {
-    gint old_base = mp_serializer_get_base(equation->priv->serializer);
+    g_return_if_fail(equation != NULL);
 
-    if (old_base == base)
+    if (mp_serializer_get_base(equation->priv->serializer) == base)
         return;
 
     mp_serializer_set_base(equation->priv->serializer, base);
-    reformat_display(equation, old_base);
+    reformat_display(equation);
     g_object_notify(G_OBJECT(equation), "base");
 }
 
@@ -600,6 +617,7 @@ math_equation_set_base(MathEquation *equation, gint base)
 gint
 math_equation_get_base(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, 10);
     return mp_serializer_get_base(equation->priv->serializer);
 }
 
@@ -607,8 +625,11 @@ math_equation_get_base(MathEquation *equation)
 void
 math_equation_set_word_size(MathEquation *equation, gint word_size)
 {
+    g_return_if_fail(equation != NULL);
+
     if (equation->priv->word_size == word_size)
         return;
+
     equation->priv->word_size = word_size;
     g_object_notify(G_OBJECT(equation), "word-size");
 }
@@ -617,6 +638,7 @@ math_equation_set_word_size(MathEquation *equation, gint word_size)
 gint
 math_equation_get_word_size(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, 64);
     return equation->priv->word_size;
 }
 
@@ -624,8 +646,11 @@ math_equation_get_word_size(MathEquation *equation)
 void
 math_equation_set_angle_units(MathEquation *equation, MPAngleUnit angle_units)
 {
+    g_return_if_fail(equation != NULL);
+
     if (equation->priv->angle_units == angle_units)
         return;
+
     equation->priv->angle_units = angle_units;
     g_object_notify(G_OBJECT(equation), "angle-units");
 }
@@ -634,6 +659,7 @@ math_equation_set_angle_units(MathEquation *equation, MPAngleUnit angle_units)
 MPAngleUnit
 math_equation_get_angle_units(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, MP_DEGREES);
     return equation->priv->angle_units;
 }
 
@@ -641,6 +667,9 @@ math_equation_get_angle_units(MathEquation *equation)
 void
 math_equation_set_source_currency(MathEquation *equation, const gchar *currency)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(currency != NULL);
+
     if (strcmp(equation->priv->source_currency, currency) == 0)
         return;
     g_free(equation->priv->source_currency);
@@ -652,6 +681,7 @@ math_equation_set_source_currency(MathEquation *equation, const gchar *currency)
 const gchar *
 math_equation_get_source_currency(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NULL);
     return equation->priv->source_currency;
 }
 
@@ -659,6 +689,9 @@ math_equation_get_source_currency(MathEquation *equation)
 void
 math_equation_set_target_currency(MathEquation *equation, const gchar *currency)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(currency != NULL);
+
     if (strcmp(equation->priv->target_currency, currency) == 0)
         return;
     g_free(equation->priv->target_currency);
@@ -670,6 +703,7 @@ math_equation_set_target_currency(MathEquation *equation, const gchar *currency)
 const gchar *
 math_equation_get_target_currency(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NULL);
     return equation->priv->target_currency;
 }
 
@@ -677,8 +711,12 @@ math_equation_get_target_currency(MathEquation *equation)
 void
 math_equation_set_source_units(MathEquation *equation, const gchar *units)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(units != NULL);
+
     if (strcmp(equation->priv->source_units, units) == 0)
         return;
+
     g_free(equation->priv->source_units);
     equation->priv->source_units = g_strdup(units);
     g_object_notify(G_OBJECT(equation), "source-units");
@@ -687,6 +725,7 @@ math_equation_set_source_units(MathEquation *equation, const gchar *units)
 const gchar *
 math_equation_get_source_units(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NULL);
     return equation->priv->source_units;
 }
 
@@ -694,8 +733,12 @@ math_equation_get_source_units(MathEquation *equation)
 void
 math_equation_set_target_units(MathEquation *equation, const gchar *units)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(units != NULL);
+
     if (strcmp(equation->priv->target_units, units) == 0)
         return;
+
     g_free(equation->priv->target_units);
     equation->priv->target_units = g_strdup(units);
     g_object_notify(G_OBJECT(equation), "target-units");
@@ -705,6 +748,7 @@ math_equation_set_target_units(MathEquation *equation, const gchar *units)
 const gchar *
 math_equation_get_target_units(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NULL);
     return equation->priv->target_units;
 }
 
@@ -712,6 +756,9 @@ math_equation_get_target_units(MathEquation *equation)
 void
 math_equation_set_status(MathEquation *equation, const gchar *status)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(status != NULL);
+
     if (strcmp(equation->priv->state.status, status) == 0)
         return;
 
@@ -724,6 +771,7 @@ math_equation_set_status(MathEquation *equation, const gchar *status)
 const gchar *
 math_equation_get_status(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NULL);
     return equation->priv->state.status;
 }
 
@@ -731,6 +779,7 @@ math_equation_get_status(MathEquation *equation)
 gboolean
 math_equation_is_empty(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, FALSE);
     return gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(equation)) == 0;
 }
 
@@ -740,6 +789,8 @@ math_equation_is_result(MathEquation *equation)
 {
     char *text;
     gboolean result;
+  
+    g_return_val_if_fail(equation != NULL, FALSE);
 
     text = math_equation_get_equation(equation);
     result = strcmp(text, "ans") == 0;
@@ -754,6 +805,8 @@ math_equation_get_display(MathEquation *equation)
 {
     GtkTextIter start, end;
 
+    g_return_val_if_fail(equation != NULL, NULL);
+
     gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(equation), &start, &end);
     return gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &start, &end, FALSE);
 }
@@ -768,6 +821,8 @@ math_equation_get_equation(MathEquation *equation)
     const gchar *read_iter;
     gboolean last_is_digit = FALSE;
 
+    g_return_val_if_fail(equation != NULL, NULL);
+
     text = math_equation_get_display(equation);
     eq_text = g_string_sized_new(strlen(text));
 
@@ -817,17 +872,26 @@ math_equation_get_number(MathEquation *equation, MPNumber *z)
     gchar *text;
     gboolean result;
 
-    text = math_equation_get_equation(equation);
-    result = !mp_serializer_from_string(equation->priv->serializer, text, z);
-    g_free(text);
+    g_return_val_if_fail(equation != NULL, FALSE);
+    g_return_val_if_fail(z != NULL, FALSE);
 
-    return result;
+    if (math_equation_is_result(equation)) {
+        mp_set_from_mp(math_equation_get_answer(equation), z);
+        return TRUE;
+    }
+    else {
+        text = math_equation_get_equation(equation);
+        result = !mp_serializer_from_string(equation->priv->serializer, text, z);
+        g_free(text);
+        return result;
+    }
 }
 
 
 MpSerializer *
 math_equation_get_serializer(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NULL);
     return equation->priv->serializer;
 }
 
@@ -835,6 +899,8 @@ math_equation_get_serializer(MathEquation *equation)
 void
 math_equation_set_number_mode(MathEquation *equation, NumberMode mode)
 {
+    g_return_if_fail(equation != NULL);
+
     if (equation->priv->number_mode == mode)
         return;
 
@@ -848,6 +914,7 @@ math_equation_set_number_mode(MathEquation *equation, NumberMode mode)
 NumberMode
 math_equation_get_number_mode(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, NORMAL);
     return equation->priv->number_mode;
 }
 
@@ -855,6 +922,7 @@ math_equation_get_number_mode(MathEquation *equation)
 gboolean
 math_equation_in_solve(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, FALSE);
     return equation->priv->in_solve;
 }
 
@@ -862,6 +930,7 @@ math_equation_in_solve(MathEquation *equation)
 const MPNumber *
 math_equation_get_answer(MathEquation *equation)
 {
+    g_return_val_if_fail(equation != NULL, FALSE);
     return &equation->priv->state.ans;
 }
 
@@ -870,6 +939,9 @@ void
 math_equation_store(MathEquation *equation, const gchar *name)
 {
     MPNumber t;
+  
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(name != NULL);
 
     if (!math_equation_get_number(equation, &t))
         math_equation_set_status(equation, _("No sane value to store"));
@@ -881,14 +953,17 @@ math_equation_store(MathEquation *equation, const gchar *name)
 void
 math_equation_recall(MathEquation *equation, const gchar *name)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(name != NULL);
     math_equation_insert(equation, name);
 }
 
 
 void
 math_equation_set(MathEquation *equation, const gchar *text)
-
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(text != NULL);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), text, -1);
     clear_ans(equation, FALSE);
 }
@@ -900,6 +975,9 @@ math_equation_set_number(MathEquation *equation, const MPNumber *x)
     char *text;
     GtkTextIter start, end;
 
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(x != NULL);
+
     /* Show the number in the user chosen format */
     text = mp_serializer_to_string(equation->priv->serializer, x);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), text, -1);
@@ -918,6 +996,9 @@ math_equation_set_number(MathEquation *equation, const MPNumber *x)
 void
 math_equation_insert(MathEquation *equation, const gchar *text)
 {
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(text != NULL);
+
     /* Replace ** with ^ (not on all keyboards) */
     if (!gtk_text_buffer_get_has_selection(GTK_TEXT_BUFFER(equation)) &&
         strcmp(text, "×") == 0 && equation->priv->state.entered_multiply) {
@@ -929,10 +1010,6 @@ math_equation_insert(MathEquation *equation, const gchar *text)
         return;
     }
 
-    /* Start new equation when entering digits after existing result */
-    if(math_equation_is_result(equation) && g_unichar_isdigit(g_utf8_get_char(text)))
-        gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), "", -1);
-
     /* Can't enter superscript minus after entering digits */
     if (strstr("⁰¹²³⁴⁵⁶⁷⁸⁹", text) != NULL || strcmp("⁻", text) == 0)
         equation->priv->can_super_minus = FALSE;
@@ -952,6 +1029,9 @@ math_equation_insert_digit(MathEquation *equation, guint digit)
     static const char *subscript_digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", NULL};
     static const char *superscript_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹", NULL};
 
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(digit < 16);
+
     if (equation->priv->number_mode == NORMAL || digit >= 10) {
         gchar buffer[7];
         gint len;
@@ -971,6 +1051,9 @@ math_equation_insert_numeric_point(MathEquation *equation)
 {
     gchar buffer[7];
     gint len;
+
+    g_return_if_fail(equation != NULL);
+
     len = g_unichar_to_utf8(mp_serializer_get_radix(equation->priv->serializer), buffer);
     buffer[len] = '\0';
     math_equation_insert(equation, buffer);
@@ -981,6 +1064,10 @@ void
 math_equation_insert_number(MathEquation *equation, const MPNumber *x)
 {
     char *text;
+
+    g_return_if_fail(equation != NULL);
+    g_return_if_fail(x != NULL);
+
     text = mp_serializer_to_string(equation->priv->serializer, x);
     math_equation_insert(equation, text);
     g_free(text);
@@ -990,6 +1077,7 @@ math_equation_insert_number(MathEquation *equation, const MPNumber *x)
 void
 math_equation_insert_exponent(MathEquation *equation)
 {
+    g_return_if_fail(equation != NULL);
     math_equation_insert(equation, "×10");
     math_equation_set_number_mode(equation, SUPERSCRIPT);
 }
@@ -998,6 +1086,7 @@ math_equation_insert_exponent(MathEquation *equation)
 void
 math_equation_insert_subtract(MathEquation *equation)
 {
+    g_return_if_fail(equation != NULL);
     if (equation->priv->number_mode == SUPERSCRIPT && equation->priv->can_super_minus) {
         math_equation_insert(equation, "⁻");
         equation->priv->can_super_minus = FALSE;
@@ -1175,7 +1264,7 @@ math_equation_show_in_progress(gpointer data)
 {
     MathEquation *equation = MATH_EQUATION(data);
     if (equation->priv->in_solve)
-        math_equation_set_status(equation, "Calculating");
+        math_equation_set_status(equation, _("Calculating"));
     return false;
 }
 
@@ -1217,6 +1306,8 @@ math_equation_solve(MathEquation *equation)
 {
     GError *error = NULL;
 
+    g_return_if_fail(equation != NULL);
+
     // FIXME: should replace calculation or give error message
     if (equation->priv->in_solve)
         return;
@@ -1287,6 +1378,8 @@ math_equation_factorize(MathEquation *equation)
     MPNumber x;
     GError *error = NULL;
 
+    g_return_if_fail(equation != NULL);
+
     // FIXME: should replace calculation or give error message
     if (equation->priv->in_solve)
         return;
@@ -1313,7 +1406,9 @@ void
 math_equation_delete(MathEquation *equation)
 {
     gint cursor;
-    GtkTextIter start, end;    
+    GtkTextIter start, end;
+
+    g_return_if_fail(equation != NULL);
 
     g_object_get(G_OBJECT(equation), "cursor-position", &cursor, NULL);
     if (cursor >= gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(equation)))
@@ -1328,6 +1423,8 @@ math_equation_delete(MathEquation *equation)
 void
 math_equation_backspace(MathEquation *equation)
 {
+    g_return_if_fail(equation != NULL);
+
     /* Can't delete empty display */
     if (math_equation_is_empty(equation))
         return;
@@ -1345,6 +1442,8 @@ math_equation_backspace(MathEquation *equation)
 void
 math_equation_clear(MathEquation *equation)
 {
+    g_return_if_fail(equation != NULL);
+
     math_equation_set_number_mode(equation, NORMAL);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), "", -1);
     clear_ans(equation, FALSE);
@@ -1356,6 +1455,8 @@ math_equation_shift(MathEquation *equation, gint count)
 {
     MPNumber z;
 
+    g_return_if_fail(equation != NULL);
+
     if (!math_equation_get_number(equation, &z)) {
         math_equation_set_status(equation,
                                  /* This message is displayed in the status bar when a bit
@@ -1376,6 +1477,8 @@ math_equation_toggle_bit(MathEquation *equation, guint bit)
     guint64 bits;
     gboolean result;
 
+    g_return_if_fail(equation != NULL);
+
     result = math_equation_get_number(equation, &x);
     if (result) {
         MPNumber max;
@@ -1491,7 +1594,7 @@ math_equation_get_property(GObject    *object,
         g_value_set_enum(value, self->priv->number_mode);
         break;
     case PROP_ACCURACY:
-        g_value_set_int(value, mp_serializer_get_accuracy(self->priv->serializer));
+        g_value_set_int(value, mp_serializer_get_trailing_digits(self->priv->serializer));
         break;
     case PROP_SHOW_THOUSANDS_SEPARATORS:
         g_value_set_boolean(value, mp_serializer_get_show_thousands_separators(self->priv->serializer));
@@ -1700,6 +1803,7 @@ pre_insert_text_cb(MathEquation  *equation,
                    gpointer       user_data)
 {
     gunichar c;
+    gint cursor;
   
     if (equation->priv->in_reformat)
         return;
@@ -1710,10 +1814,11 @@ pre_insert_text_cb(MathEquation  *equation,
         math_equation_push_undo_stack(equation);
 
     /* Clear result on next digit entered if cursor at end of line */
-    // FIXME Cursor
     c = g_utf8_get_char(text);
+    g_object_get(G_OBJECT(equation), "cursor-position", &cursor, NULL);
     if ((g_unichar_isdigit(c) || c == mp_serializer_get_radix(equation->priv->serializer)) &&
-         math_equation_is_result(equation)) {
+         math_equation_is_result(equation) &&
+         cursor >= gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(equation))) {
         gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), "", -1);
         clear_ans(equation, FALSE);
         gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(equation), location);
@@ -1815,7 +1920,9 @@ math_equation_init(MathEquation *equation)
     const char *digit_values = _("0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F");
     const char *default_digits[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"};
     gchar **digits;
-    gboolean use_default_digits = FALSE;
+    /* Default to using untranslated digits, this is because it doesn't make sense in most languages and we need to make this optional.
+     * See https://bugzilla.gnome.org/show_bug.cgi?id=632661 */
+    gboolean use_default_digits = TRUE;
     int i;
 
     equation->priv = G_TYPE_INSTANCE_GET_PRIVATE(equation, math_equation_get_type(), MathEquationPrivate);
